//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Bivariate2DInterpolantScattered.h --- Implementation of Interpolant2D class
//
//	DESCRIPTION:
//		Implementation of Interpolant2D class with bivariate interpolant
//		using scattered grid points - based on routine TOMS 660
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/Bivariate2DInterpolantScattered.h,v 1.2 2002/12/17 17:10:13 jbakker Exp $
//

#ifndef BIVARIATE_TWOD_INTERPOLANT_SCATTERED_H
#define BIVARIATE_TWOD_INTERPOLANT_SCATTERED_H

#include "Interpolant2D.h"

namespace Slatec {

template<class NumericType>
class Bivariate2DInterpolantScattered :
							public virtual Interpolant2D<NumericType> {
public:
	Bivariate2DInterpolantScattered(const std::vector<NumericType> &x,
								    const std::vector<NumericType> &y,
									const std::vector<NumericType> &z);
		//
		//	PURPOSE:
		//		Instantiate a Bivariate2DInterpolantScattered object
		//
		//	ARGUMENTS:
		//		x/y	: vectors of independent variable values
		//		z	: vector of dependent variable values corresponding to
		//			  <x/y>
		//

	NumericType operator()(NumericType x, NumericType y) const;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <x,y>
		//


	void additionalDataPoints(unsigned nPoints = 4);
		//
		//	PURPOSE:
		//		Set the number of points that will be used to approximate
		//		partial derivatives
		//


	~Bivariate2DInterpolantScattered();
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//

private:
	NumericType	*_xd, *_yd,		//	cartesian coordinates of nodes
			    *_f;			//	values at nodes
	int			_n,				//	number of data points _xd/_yd
				_nq,			//	number of data points to be used in
								//	least square fit for coefficients
				_nw,			//	number of nodes within (and defining) the
								//	radii of influence r(k) which enter into
								//	the weights
				_nr;			//	number of rows and columns in the cell grid
	int			*_lcell,		//	array of length >= _nr**2.
				*_lnext;		//	array of length >= _n
	NumericType	*_rsq,			//	array of length >= _n
				*_a;			//	array of length >= 5*_n

	NumericType	_xmin, _ymin,
				_dx, _dy,
				_rmax;
};

}

#endif
