//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CubicSplineInterpolant.h --- Implementation of Interpolant class as
//								  cubic Spline functions
//
//	DESCRIPTION:
//		A CubicSplineInterpolant object represents a monotone piecewise
//		cubic spline interpolant to given data.
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//		UL, 2000-02-01: 1.1.0: - added integral()
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/CubicSplineInterpolant.h,v 1.8 2001/11/06 17:07:22 ulammers Exp $
//

#ifndef CUBIC_Spline_INTERPOLANT_H
#define CUBIC_Spline_INTERPOLANT_H

#include "Interpolant.h"

namespace Slatec {

template<class NumericType>
class CubicSplineInterpolant : public Interpolant<NumericType> {
public:
	CubicSplineInterpolant(const std::vector<NumericType> &x,
						   const std::vector<NumericType> &y);
		//
		//	PURPOSE:
		//		Instantiate a CubicSplineInterpolant object
		//
		//	ARGUMENTS:
		//		x	: vector of independent variable values
		//		y	: vector of dependent variable values corresponding to <X>
		//


	NumericType operator()(NumericType x) const;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <X>
		//


	const Interpolant<NumericType> &operator*=(NumericType);
	const Interpolant<NumericType> &operator*=
									(const Interpolant<NumericType> &);
	const Interpolant<NumericType> &operator+=
									(const Interpolant<NumericType> &);
	const Interpolant<NumericType> &operator+=(NumericType);
		//
		//	PURPOSE:
		//		Perform arithmetic on CubicHermiteInterpolant
		//


	Interpolant<NumericType> &supplement(const std::vector<NumericType> &xsupp,
									 const std::vector<NumericType> &ysupp);
		//
		//	PURPOSE:
		//		Supplement interpolated curve by additional	data pairs
		//		in <xsupp>/<ysupp> vectors
		//


	NumericType integral(NumericType x1, NumericType x2) const;
		//
		//	PURPOSE:
		//		Calculate integral of interpolant between <x1> and <x2>
		//


	~CubicSplineInterpolant();
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//

private:
	unsigned	_n;
	NumericType	*_x, *_y, *_deriv;
	NumericType	_f, _a;

	void initSlatec();
	void initFromVectors(const std::vector<NumericType> &x,
						 const std::vector<NumericType> &y);
	void free();
};

}

#endif
