//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Interpolant.h --- A class representing an interpolation function
//
//	DESCRIPTION:
//		Instances of class Interpolant represent 1D-functions which are
//		defined by a discrete set of X/Y data points. The function
//		is evaluated at intermediate data points by interpolation.
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//		UL, 2000-02-01: 1.1.0: - added integral() method
//		UL, 2000-03-13: 1.2.0: - added abscissa() method + d/f constructor
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/Interpolant.h,v 1.9 2002/12/17 17:10:13 jbakker Exp $
//

#ifndef INTERPOLANT_H
#define INTERPOLANT_H

#include <vector>
#include "Curve.h"

namespace Slatec {

template<class NumericType>
class Interpolant : public Curve<NumericType> {
public:
	Interpolant() {}
	Interpolant(const std::vector<NumericType> &abscissa) : _abscissa(abscissa) {}
		//
		//	PURPOSE:
		//		Instantiate an Interpolant object
		//

	const std::vector<NumericType> &abscissa() const { return _abscissa; }
		//
		//	PURPOSE:
		//		Return abscissa on which the Interpolant is defined
		//


	virtual Interpolant<NumericType> &supplement(
									const std::vector<NumericType> &,
									const std::vector<NumericType> &) = 0;
		//
		//	PURPOSE:
		//		Supplement object by additional data points
		//


	virtual ~Interpolant() {}
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//

protected:
	std::vector<NumericType>	_abscissa;

};

}

#endif
