//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Interpolant2D.h --- A class representing an 2-D interpolation function
//
//	DESCRIPTION:
//		Instances of class Interpolant2D represent 2D-functions which are
//		defined by a discrete set of X/Y/Z data points. The function
//		is evaluated at intermediate data points by interpolation.
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/Interpolant2D.h,v 1.1 2000/01/13 10:19:16 ulammers Exp $
//

#ifndef INTERPOLANT_TWOD_H
#define INTERPOLANT_TWOD_H

#include <vector>

namespace Slatec {

template<class NumericType>
class Interpolant2D {
public:
	Interpolant2D() {}
		//
		//	PURPOSE:
		//		Instantiate an Interpolant2D object
		//

	virtual NumericType operator()(NumericType x, NumericType) const = 0;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <X,Y>
		//

	virtual ~Interpolant2D() {}
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//
};

}

#endif
