//
//	test Bivariate2DInterpolantRect
//
//	UL, Wed Jan 12 16:43:52 MET 2000
//

#include <iostream>
#include <vector>
#include <cstdlib>
#include <cmath>
#include<cassert>
#include "slatec++.h"
#include "common.h"

namespace {
bool isClose(double a, double b)
{
	if (a == 0.)
		return fabs(b) < 1.e-10;

    return fabs(a)/fabs(b)-1. < 1.e-4;
}

static float surface(float x, float y) { return x+3*y; }

}



int main()
{
	const unsigned N = 50;
	std::vector<float> x(N), y(N), z(N*N);

	//	test with scattered grid points
	float fg[N][N];

	for (unsigned i=0; i<N; ++i) {
		x[i] = FLT(i);
		y[i] = FLT(i);
	}

	for (unsigned i=0; i<N; ++i)
		for (unsigned j=0; j<N; ++j)
			fg[j][i] = surface(x[i], y[j]);

	float *p = fg[0];
	for (unsigned n=0; n<N*N; ++n)
		z[n] = p[n];
	
	Slatec::Bivariate2DInterpolantRectangular<float> f(x, y, z);

  	for(unsigned i=0; i<N; i++) {
		//	test 100 points
		double x = 20.*FLT(rand())/RAND_MAX,
			   y = 20.*FLT(rand())/RAND_MAX,
			   z0 = surface(x, y),
			   z1 = f(x, y);

		std::cout<<"f("<<x<<", "<<y<<")= "<<z0<<" - interpolated: "<<z1<<std::endl;
		assert(isClose(z0, z1));		
  	}

	return 0;
}

